package geniusweb.javavenv;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.util.Arrays;

import org.junit.Test;

public class PythonCallTest {
	@Test(expected = IllegalArgumentException.class)
	public void testSmokeNullPython() {
		new Call(null, Arrays.asList("troep"), 1000);
	}

	@Test
	public void testSmokeNoSuchPython() {
		new Call("/a/c/e", Arrays.asList("troep"), 1000);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testSmokeEmptyArg() {
		new Call("/usr/local/bin/python3", null, 1000);
	}

	@Test
	public void testSmoke() {
		new Call("/usr/local/bin/python3", Arrays.asList("troep"), 1000);
	}

	@Test
	public void testNonsenseCall()
			throws InterruptedException, IOException, PythonError {
		try {
			Call c = new Call("/usr/local/bin/python3",
					Arrays.asList("troep"), 1000);
			c.run();
		} catch (PythonError e) {
			System.out.println("Correctly received Pythonerror:" + e);
			return;
		}
		throw new IllegalStateException("Expected error did not occur");
	}

	@Test
	public void testPrint()
			throws InterruptedException, IOException, PythonError {
		Call c = new Call("/usr/local/bin/python3",
				Arrays.asList("src/test/resources/print.py"), 1000);
		assertEquals("hello\n", c.run());
	}

	@Test
	public void testPrintNoNewline()
			throws InterruptedException, IOException, PythonError {
		Call c = new Call("/usr/local/bin/python3",
				Arrays.asList("src/test/resources/printnonewline.py"), 1000);
		assertEquals("hello", c.run());
	}

	@Test
	public void testPrint3xNoNewline()
			throws InterruptedException, IOException, PythonError {
		Call c = new Call("/usr/local/bin/python3",
				Arrays.asList("src/test/resources/print3xnonewline.py"), 1000);
		// the call collects till the connection closes, also if it's multi
		// lines
		assertEquals("hellothis is another messageAnd another", c.run());
	}

	@Test
	public void testWait5Sec()
			throws InterruptedException, IOException, PythonError {
		Call c = new Call("/usr/local/bin/python3",
				Arrays.asList("src/test/resources/wait5sec.py"), 10000);
		assertEquals("hello world\n", c.run());
	}

	@Test
	public void testWait5SecInHurry()
			throws InterruptedException, IOException, PythonError {
		try {
			Call c = new Call("/usr/local/bin/python3",
					Arrays.asList("src/test/resources/wait5sec.py"), 1000);
			c.run();
		} catch (InterruptedException e) {
			System.out.println("Correctly received InterruptedException:" + e);
			return;
		}
		throw new IllegalStateException("Expected error did not occur");
	}

	@Test
	public void testPrintErr()
			throws InterruptedException, IOException, PythonError {
		try {
			Call c = new Call("/usr/local/bin/python3",
					Arrays.asList("src/test/resources/printerr.py"), 1000);
			c.run();
		} catch (PythonError e) {
			System.out.println("Correctly received Pythonerror:" + e);
			return;
		}
		throw new IllegalStateException("Expected error did not occur");
	}

	@Test
	public void testThrow()
			throws InterruptedException, IOException, PythonError {
		try {
			Call c = new Call("/usr/local/bin/python3",
					Arrays.asList("src/test/resources/throw.py"), 1000);
			c.run();
		} catch (PythonError e) {
			System.out.println("Correctly received Pythonerror:" + e);
			return;
		}
		throw new IllegalStateException("Expected error did not occur");
	}

}
